/* Copyright (c) 2009 The Regents of the Signet Group.
*  All rights reserved.
*
*  Permission to use, copy, modify, and distribute this software and its
*  documentation for any purpose, without fee, and without written
*  agreement is hereby granted, provided that the above copyright
*  notice, the (updated) modification history and the author appear in
*  all copies of this source code.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS `AS IS'
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
*  ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS
*  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
*  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, LOSS OF USE, DATA,
*  OR PROFITS) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
*  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
*  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
*  THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef FLASH_MANAGER_MESSAGES_H
#define FLASH_MANAGER_MESSAGES_H

#ifndef TOSH_DATA_LENGTH
#define TOSH_DATA_LENGTH 100
#warning Message OK
#endif


enum{
		AM_SERIAL_CMD_OFFSET             = 120,
		AM_FLASHMANAGERMESSAGEFORMAT     = AM_SERIAL_CMD_OFFSET+1,
		AM_FLASHMANAGERMESSAGEWRITE      = AM_SERIAL_CMD_OFFSET+2,
		AM_FLASHMANAGERMESSAGETABLE      = AM_SERIAL_CMD_OFFSET+3,
		AM_FLASHMANAGERMESSAGEREAD       = AM_SERIAL_CMD_OFFSET+5,
		
		DATA_SIZE 						 = 80, //Size of data chunk sent in a Write packet
};

// Format flash command
/* Message received from node: Format volume
 * Message sended   from node: Format volume done
 */
typedef nx_struct FlashManagerMessageFormat{
}FlashManagerMessageFormat;

// Write to flash command
/* Message received from node: 
 * 		Write "size" bytes of "data" buffer at address "address"
 * 		Sender must send a Write command with "size"==0 as the last command, 
 * 		to make permanent write operations with an internal call to sync()  
 * Message sended   from node: 
 * 		Write done, receiver must ignore message fields
 */
typedef nx_struct FlashManagerMessageWrite{
	nx_uint32_t address;
	nx_uint32_t size;
	nx_uint8_t data[DATA_SIZE];	
}FlashManagerMessageWrite;


// Write to flash command
enum{
	CMD_GET_TABLE=1,
	CMD_ADD_ENTRY=2,
};
/* Message received from node, case cmd: 
 * 		CMD_GET_TABLE : Ask for the partition table
 *  	CMD_ADD_ENTRY : Add the entry specified to the partition table
 * Message sended   from node, case cmd: 
 * 		CMD_GET_TABLE : The entry specified is present in the partition table
 *  	CMD_ADD_ENTRY : The entry has been added
 */ 
typedef nx_struct FlashManagerMessageTable{
	nx_uint16_t flags;
    nx_uint16_t hashName;
    nx_uint32_t startAddr; 			
    nx_uint32_t size;
    nx_uint16_t progStartOffs;
    nx_uint32_t spaceLeft;
    
    nx_uint8_t  cmd;       			
}FlashManagerMessageTable;

typedef nx_struct FlashManagerMessageRead{
	nx_uint32_t address;
	nx_uint32_t size;
	nx_uint8_t data[DATA_SIZE];	
}FlashManagerMessageRead;

#endif /* FLASH_MANAGER_MESSAGES_H */
