/* Copyright (c) 2009 The Regents of the Signet Group.
*  All rights reserved.
*
*  Permission to use, copy, modify, and distribute this software and its
*  documentation for any purpose, without fee, and without written
*  agreement is hereby granted, provided that the above copyright
*  notice, the (updated) modification history and the author appear in
*  all copies of this source code.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS `AS IS'
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
*  ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS
*  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
*  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, LOSS OF USE, DATA,
*  OR PROFITS) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
*  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
*  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
*  THE POSSIBILITY OF SUCH DAMAGE.
*/


#ifndef SYNAPSE__STATISTICS_H
#define SYNAPSE__STATISTICS_H

#ifndef SYNAPSE_STATISTICS
#define SYNAPSE_STATISTICS FALSE
#endif

#include "current_task.h"
#include "Synapse_msgs.h"
#include "noinline.h"

enum{
	AM_SYNAPSESTATISTICSMSG=AM_SYNAPSE_SERIAL_STATS, // For MIG
};

enum{
	STATFLAG_CURRUPT= 1 << 0,
};


/*
 * Lot of ROM space is saved using a standard struct for processing statistics,
 * this struct is finally copied field by field into a nx_struct before sending.
 * 
 * *** ADD EVERY FIELD YOU ADD HERE ALSO TO THE NORMAL STRUCT AND TO THE COPY METHOD *** 
 */
typedef nx_struct SynapseStatisticsMsg{
    nx_uint16_t adv_tx;
    nx_uint16_t req_tx;
    nx_uint16_t data_tx;
    nx_uint16_t cycles_for_rx;
    nx_uint16_t cycles_for_rx_completion;
//    nx_uint16_t fixed_rx;
//    nx_uint16_t failed_rx;
    nx_uint16_t correct_advs;
    nx_uint16_t corrupt_advs;
    nx_uint16_t unsync_advs;
    nx_uint16_t correct_reqs;
    nx_uint16_t corrupt_reqs;
    nx_uint16_t unsync_reqs;
    nx_uint16_t correct_datas;
    nx_uint16_t corrupt_datas;
    nx_uint8_t  correct_blocks;
    nx_uint16_t corrupt_blocks;   
    nx_uint8_t  min_hopcount;
    nx_uint8_t  max_hopcount;
    nx_uint16_t flags;
    nx_uint16_t missed_sync;
    nx_uint16_t success_sync;
    nx_int16_t  avg_sync_offset;
} SynapseStatisticsMsg;

typedef struct SynapseStatisticsMsgQuick{
    uint16_t adv_tx;
    uint16_t req_tx;
    uint16_t data_tx;
    uint16_t cycles_for_rx;
    uint16_t cycles_for_rx_completion;
//    nx_uint16_t fixed_rx;
//    nx_uint16_t failed_rx;
    uint16_t correct_advs;
    uint16_t corrupt_advs;
    uint16_t unsync_advs;
    uint16_t correct_reqs;
    uint16_t corrupt_reqs;
    uint16_t unsync_reqs;
    uint16_t correct_datas;
    uint16_t corrupt_datas;
    uint8_t  correct_blocks;
    uint16_t corrupt_blocks;   
    uint8_t  min_hopcount;
    uint8_t  max_hopcount;
    uint16_t flags;
    uint16_t missed_sync;
    uint16_t success_sync;
    int16_t  avg_sync_offset;
} SynapseStatisticsMsgQuick;

SynapseStatisticsMsgQuick synapseStats;
SynapseStatisticsMsg synapseStatsMsg;
bool synapseStatsActive=FALSE;

#define setSynapseStat(stat,val) {if(SYNAPSE_STATISTICS) stat = val ;}
#define incSynapseStat(stat    ) {if(SYNAPSE_STATISTICS) stat ++ ;}
#define minSynapseStat(stat,val) {if(SYNAPSE_STATISTICS) stat = ( val < stat )? val : stat ;}
#define maxSynapseStat(stat,val) {if(SYNAPSE_STATISTICS) stat = ( val > stat )? val : stat ;}




void copyStatsRecord(){
	synapseStatsMsg.adv_tx=synapseStats.adv_tx;
	synapseStatsMsg.req_tx=synapseStats.req_tx;
	synapseStatsMsg.data_tx=synapseStats.data_tx;
	synapseStatsMsg.cycles_for_rx=synapseStats.cycles_for_rx;
	synapseStatsMsg.cycles_for_rx_completion=synapseStats.cycles_for_rx_completion;
//	synapseStatsMsg.fixed_rx=synapseStats.fixed_rx;
//	synapseStatsMsg.failed_rx=synapseStats.failed_rx;
	synapseStatsMsg.correct_advs=synapseStats.correct_advs;
	synapseStatsMsg.corrupt_advs=synapseStats.corrupt_advs;
	synapseStatsMsg.unsync_advs=synapseStats.unsync_advs;
	synapseStatsMsg.correct_reqs=synapseStats.correct_reqs;
	synapseStatsMsg.corrupt_reqs=synapseStats.corrupt_reqs;
	synapseStatsMsg.unsync_reqs=synapseStats.unsync_reqs;
	synapseStatsMsg.correct_datas=synapseStats.correct_datas;
	synapseStatsMsg.corrupt_datas=synapseStats.corrupt_datas;
	synapseStatsMsg.correct_blocks=synapseStats.correct_blocks;
	synapseStatsMsg.corrupt_blocks=synapseStats.corrupt_blocks;
	synapseStatsMsg.min_hopcount=synapseStats.min_hopcount;
	synapseStatsMsg.max_hopcount=synapseStats.max_hopcount;
	synapseStatsMsg.flags=synapseStats.flags;
	synapseStatsMsg.missed_sync=synapseStats.missed_sync;
	synapseStatsMsg.success_sync=synapseStats.success_sync;
	synapseStatsMsg.avg_sync_offset=synapseStats.avg_sync_offset;  
}




 
void startSynapseStats(){
    if(SYNAPSE_STATISTICS){
    	if(!synapseStatsActive){
    		synapseStatsActive=TRUE;
            memset(&synapseStats, 0, sizeof(SynapseStatisticsMsg));
            synapseStats.min_hopcount=0xFF;
        }
    }	
}
void sendSynapseStats();
NOINLINE void stopSynapseStats(){
	if(SYNAPSE_STATISTICS){
		if(synapseStatsActive){
            synapseStats.avg_sync_offset=synapseStats.avg_sync_offset/(int16_t)synapseStats.success_sync;
            
			synapseStatsActive=FALSE;
			copyStatsRecord();
			sendSynapseStats();
		}
	}
}

#endif /* SYNAPSE__STATISTICS_H */
