/* Copyright (c) 2009 The Regents of the Signet Group.
*  All rights reserved.
*
*  Permission to use, copy, modify, and distribute this software and its
*  documentation for any purpose, without fee, and without written
*  agreement is hereby granted, provided that the above copyright
*  notice, the (updated) modification history and the author appear in
*  all copies of this source code.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS `AS IS'
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
*  ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS
*  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
*  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, LOSS OF USE, DATA,
*  OR PROFITS) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
*  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
*  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
*  THE POSSIBILITY OF SUCH DAMAGE.
*/

package net.tinyos.signet;

import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.Random;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import com.sun.org.apache.xml.internal.serialize.XMLSerializer;

public class Key {
	protected byte[] value;
	protected int index;
	
	public Key(int index){
		this.index=index;			
	}		
	public Key(int index,int length){
		this(index);
		value=new byte[length];	
		for(int i=0;i<value.length;i++)value[i]=0;
	}
	public Key(int index,byte[] value){
		this(index);
		this.value=value.clone();			
	}
	public Key(int index,String value){
		this(index);		
		valueFromString(value);
	}
	
	public void generate(int length){
		value=new byte[length];	
		new Random().nextBytes(value);		
	}
	public void generate(){
		generate(value.length);
	}
	
	private void valueFromString(String s){
		this.value=new byte[s.length()/2];
		for(int i=0;i<s.length();i+=2)
			this.value[i/2]=(byte)Integer.parseInt(s.substring(i, i+2), 16);		
	}
	
	public static String valueToString(byte[] value){
		String result="";
		for(int i=0;i<value.length;i++){
			String s="0"+Integer.toHexString(value[i]).toUpperCase();
			result+=s.substring(s.length()-2, s.length());
		}
		return result;		
	}
	
	public String valueToString(){
		return valueToString(this.value);
	}
	
	public void saveToXML(Element container) throws ParserConfigurationException{
		Element root=container;
		Element eindex 	=container.getOwnerDocument().createElement(	"index");
		Element evalue 	=container.getOwnerDocument().createElement(	"value");		
										
		root	.appendChild(eindex);
		root	.appendChild(evalue);
		
		eindex.appendChild(container.getOwnerDocument().createTextNode(Integer.toString(index)));
		evalue.appendChild(container.getOwnerDocument().createTextNode(valueToString()));				
	}
	
	public void loadFromXML(Element container) throws ParserConfigurationException{
		Document doc=container.getOwnerDocument();
		index=Integer.parseInt(
				container.getElementsByTagName("index").item(0).getTextContent()
				);
		valueFromString(
				container.getElementsByTagName("value").item(0).getTextContent()
				);					
	}
	
	public static Key createFromXML(Element container) throws ParserConfigurationException{
		Key result=new Key(0);
		result.loadFromXML(container);
		return result;
	}

	public static void main(String[] args) throws IOException, ParserConfigurationException {
		Document doc =DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
		Element root=doc.createElement("Main");
		doc.appendChild(root);
		
		Key k=new Key(0,10);
		
		k.saveToXML(root);
		
		XMLSerializer xs=new XMLSerializer();
		xs.setOutputCharStream(new OutputStreamWriter(System.out));
		xs.serialize(doc);	
		
		k=Key.createFromXML(root);
		
		doc =DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
		root=doc.createElement("Main");
		doc.appendChild(root);
		
		k.saveToXML(root);
		xs.serialize(doc);			
	}
}

