/* Copyright (c) 2009 The Regents of the Signet Group.
*  All rights reserved.
*
*  Permission to use, copy, modify, and distribute this software and its
*  documentation for any purpose, without fee, and without written
*  agreement is hereby granted, provided that the above copyright
*  notice, the (updated) modification history and the author appear in
*  all copies of this source code.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS `AS IS'
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
*  ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS
*  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
*  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, LOSS OF USE, DATA,
*  OR PROFITS) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
*  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
*  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
*  THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __TTIMESA_H__
#define __TTIMESA_H__

// Number of bits of security. 
// This can be defined in makefile. 
#ifndef SECURITY_BITS
    #define SECURITY_BITS 80
    #define SECURITY_BITS_LOG 7
#endif     

// Abilitation for fault tolerance implementation
#ifdef FAULT_TOLERANCE
#endif


//constants
enum {
  SIG_SEC_PARAM = SECURITY_BITS/8,    // TTimeSA security parameter n in bytes
//  SIG_T = 30,                         // TTimeSA parameter T
  SIG_KEY_SIZE = SIG_SEC_PARAM,       // Size of the keys in bytes used in TTimeSA
  SIG_CHECKSUM_BITSIZE = SECURITY_BITS_LOG,   // Number of bits for representing number of zeros in the hash part of the sign.
    
  /* Maximum number of signatures missed that is tollerated by verifier.
   * This is implemented allowing for SIG_MAX_MISSED_SIGNS hashes of private key
   * to be done to match the key stored in the node. This way if a node miss a 
   * dissemination session and goes out of sync with base station, it can 
   * anyway verify the signature and resync at the next dissemination session.
   * Increasing this parameter decreases security of authentication system.
   * So low values must be used, e.g. < 5. 
   */
#ifdef FAULT_TOLERANCE   
  SIG_MAX_MISSED_SIGNS=3,
#else
  SIG_MAX_MISSED_SIGNS=0,  
#endif
      
  SIG_BITVEC_SIZE = SIG_SEC_PARAM + (((SIG_CHECKSUM_BITSIZE % 8)==0)? SIG_CHECKSUM_BITSIZE/8 : SIG_CHECKSUM_BITSIZE/8+1) ,  
  SIG_BITVEC_BITSIZE=8*SIG_SEC_PARAM + SIG_CHECKSUM_BITSIZE,
  
  SIG_KEYS_OFFSET=3,
};

#endif

