/* Copyright (c) 2009 The Regents of the Signet Group.
*  All rights reserved.
*
*  Permission to use, copy, modify, and distribute this software and its
*  documentation for any purpose, without fee, and without written
*  agreement is hereby granted, provided that the above copyright
*  notice, the (updated) modification history and the author appear in
*  all copies of this source code.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS `AS IS'
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
*  ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS
*  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
*  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, LOSS OF USE, DATA,
*  OR PROFITS) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
*  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
*  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
*  THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef DEBUG_PRINTF_H
#define DEBUG_PRINTF_H

/**
 * This header allows to enable and disable globally printf debug output, by defining or not DBG macro.
 * It also allows to enable and disable more granularly printf debug, by defining other macros.
 * 
 * Example:
 * 
 *  code:
 *      DBG(DBGFLAG_VERBOSE,("Doing something"));
 *      DBG(DBGFLAG_STANDARD,("Result: %i",result));
 * 
 *  output when
 *      #define DEBUG
 *      #define DBG_FLAG_STANDARD=TRUE
 *      #define DBG_FLAG_VERBOSE=FALSE
 * 
 *      Result: 0
 *
 *  output when
 *      #define DEBUG
 *      #define DBG_FLAG_STANDARD=TRUE
 *      #define DBG_FLAG_VERBOSE=TRUE
 * 
 *      Doing something
 *      Result: 0
 *       
 * When DEBUG is not defined, no code is generated in place of DBG macro, and printf.h is not included
 * 
 */


#ifdef DEBUG
    #warning Printf debug active
    
    #include "printf.h"
    
    /**
     * When macro "flag" is defined TRUE, printf is executed, with parameter list "printf_args"
     */
    #define  DBG( flag , printf_args )  { if(flag){ printf printf_args ; printfflush();} }
    /**
     * When macro "flag" is defined TRUE, printarray is executed, with parameter list "printarray_args"
     */    
    #define  DBG_ARRAY( flag , printarray_args ) { if(flag){ __printarray printarray_args ; printfflush();} }
    
    /**
     * Prints the content of an uint8_t array, with items separated by "," and array enclosed in "[ ]"
     */
    void __printarray(uint8_t* buf,uint16_t len){
        uint16_t i;
        printf("[%i",buf[0]);
        for(i=1;i<len;i++){
            printf(",%i",buf[i]);          
        }
        printf("]");
    }    
#else
    #define  DBG( flag , printf_args )
    #define  DBG_ARRAY( flag , printarray_args )
       
    int __attribute__((format (printf, 1, 2))) printf(const char *string, ...){
    	return 0;    	
    }

#endif


#endif /* DEBUG_PRINTF_H */
