/* Copyright (c) 2009 The Regents of the Signet Group.
*  All rights reserved.
*
*  Permission to use, copy, modify, and distribute this software and its
*  documentation for any purpose, without fee, and without written
*  agreement is hereby granted, provided that the above copyright
*  notice, the (updated) modification history and the author appear in
*  all copies of this source code.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS `AS IS'
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
*  ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS
*  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
*  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, LOSS OF USE, DATA,
*  OR PROFITS) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
*  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
*  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
*  THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __SYNAPSE_BOOTLOADER_H__
#define __SYNAPSE_BOOTLOADER_H__

#define IF_OPCODE_ADDR          0x1000
#define IF_IMG2LOAD_ADDR        0x1002  
#define IF_ID_ADDR              0x1002  
#define IF_SIZE_ADDR            0x1004  
#define IF_SN_ADDR              0x1006  
#define IF_NODE_ID_ADDR         0x1008	// node ID
#define IF_PROTECTED_FLAG				0x100A	// address protected flag

sfrw(opcode, IF_OPCODE_ADDR);
sfrw(imgToLoad, IF_IMG2LOAD_ADDR);
sfrw(appID, IF_ID_ADDR);
sfrw(imgSize, IF_SIZE_ADDR);
sfrw(if_sn, IF_SN_ADDR);
sfrw(ifNodeId, IF_NODE_ID_ADDR);
sfrw(ifNotProtected, IF_PROTECTED_FLAG);

enum SynapseBootloaderCommands_enum {
    SBC_NONE            = 0,
    SBC_FORMAT          = 1,
    SBC_STORE           = 2,
    SBC_LOAD_ID         = 3,
    SBC_LOAD_PARTITION  = 4,
    SBC_INTERNAL_FORMAT = 15,
    SBC_ONLY_FORMAT     = 16,
    SBC_FORMAT_SPECIAL	= 17  			//format and store golden image
};
#define PROG_FLASH_SIZE				0xAFFF		//IF_SIZE-0x5000 

#define GOLDEN_IMAGE_SIZE			45056u 		//40000 good margin
#warning The bootloader assumes that the Golden Image will be at most 43000 bytes
#define GOLDEN_IMAGE_ID				0x36B9

void reboot() {
  //Enable watchdog
  WDTCTL = WDT_ARST_1_9;
  while(1);
}

#endif // __SYNAPSE_BOOTLOADER_H__
