/* Copyright (c) 2009 The Regents of the Signet Group.
*  All rights reserved.
*
*  Permission to use, copy, modify, and distribute this software and its
*  documentation for any purpose, without fee, and without written
*  agreement is hereby granted, provided that the above copyright
*  notice, the (updated) modification history and the author appear in
*  all copies of this source code.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS `AS IS'
*  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
*  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
*  ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS
*  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
*  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, LOSS OF USE, DATA,
*  OR PROFITS) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
*  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
*  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
*  THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "internal_flash.h"
#include "asm_utils.h"

#ifdef BOOTLOADER
void internalFlash_erase(uint8_t *startAddr, uint16_t size)
{
    uint16_t i;
    for(i=0; i<size; i+=IF_MIN_MEM_ERASE_SIZE) {
        FCTL2 = FWKEY + FSSEL1 + FN2;
        FCTL3 = FWKEY;
        FCTL1 = FWKEY + ERASE; //segment erase
        startAddr[i] = 0;
        FCTL1 = FWKEY;
        FCTL3 = FWKEY + LOCK;
    }
    FCTL2 = FWKEY + FSSEL1 + FN2;
    FCTL3 = FWKEY;
    FCTL1 = FWKEY + ERASE; //segment erase
    startAddr[size-1] = 0;
    FCTL1 = FWKEY;
    FCTL3 = FWKEY + LOCK;
}

void internalFlash_write8(volatile uint8_t *addr, uint8_t val)
{
    FCTL2 = FWKEY + FSSEL1 + FN2;
    FCTL3 = FWKEY;
    FCTL1 = FWKEY + WRT;
    *addr = val;
    FCTL1 = FWKEY;
    FCTL3 = FWKEY + LOCK;
}

void internalFlash_write16(volatile uint16_t *addr, uint16_t val)
{
    FCTL2 = FWKEY + FSSEL1 + FN2;
    FCTL3 = FWKEY;
    FCTL1 = FWKEY + WRT;
    *addr = val;
    FCTL1 = FWKEY;
    FCTL3 = FWKEY + LOCK;
}
#endif

void internalFlash_writeSegment(uint8_t inf, uint8_t offset, uint16_t *data, uint8_t len)
{
    uint16_t infData[IF_MIN_INF_ERASE_SIZE16];
    uint8_t i;
    volatile uint16_t *p;
    if(offset>0x80) {
        return;
    }
    if(inf == 0) {  //segment A
        p = (uint16_t *)IF_SEGMENT_A_ADDR;
    } else {    //segment B
        p = (uint16_t *)IF_SEGMENT_B_ADDR;
    }
    if(offset+len>IF_MIN_INF_ERASE_SIZE16) {
        len = IF_MIN_INF_ERASE_SIZE16-offset;
    }
    // save data
    for(i=0; i<IF_MIN_INF_ERASE_SIZE16; i++) {
        infData[i] = p[i];
    }
    // update data
    for(i=0; i<len; i++) {
        infData[offset+i] = data[i];
    }
    // clear segment
    dint();
    FCTL2 = FWKEY + FSSEL1 + FN2;
    FCTL3 = FWKEY;
    FCTL1 = FWKEY + ERASE;
    p[0] = 0;
    FCTL1 = FWKEY;
    FCTL3 = FWKEY + LOCK;
    // store data
    for(i=0; i<IF_MIN_INF_ERASE_SIZE16; i++) {
        FCTL2 = FWKEY + FSSEL1 + FN2;
        FCTL3 = FWKEY;
        FCTL1 = FWKEY + WRT;
        p[i] = infData[i];
        FCTL1 = FWKEY;
        FCTL3 = FWKEY + LOCK;
    }
    eint();
}
